@@ -1,4 +1,5 @@
-// +build nrf
+//go:build nrf && !nrf52840
+// +build nrf,!nrf52840
 
 package runtime
 
@@ -27,7 +28,7 @@ func main() {
 }
 
 func init() {
-	machine.Serial.Configure(machine.UARTConfig{})
+	machine.InitSerial()
 	initLFCLK()
 	initRTC()
 }
@@ -65,6 +66,18 @@ func putchar(c byte) {
 	machine.Serial.WriteByte(c)
 }
 
+func getchar() byte {
+	for machine.Serial.Buffered() == 0 {
+		Gosched()
+	}
+	v, _ := machine.Serial.ReadByte()
+	return v
+}
+
+func buffered() int {
+	return machine.Serial.Buffered()
+}
+
 func sleepTicks(d timeUnit) {
 	for d != 0 {
 		ticks := uint32(d) & 0x7fffff // 23 bits (to be on the safe side)
